/***************************************************************************
                          icesndcfg.cpp  -  description
                             -------------------
    begin                :   13 23:04:12 EEST 2003
    copyright            : (C) 2003 by Vadim A. Khohlov
    email                : Vadim@localhost.localdomain
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qlistbox.h>
#include <qdir.h>
#include <qfiledialog.h>
#include <qfileinfo.h>
#include <qtooltip.h>

#include <stdlib.h>

#include "icesndcfg.h"
#define GUI_EVENT_NAMES
#include "guievent.h"

IcesndcfgApp::IcesndcfgApp():
icewmCfgDir(getenv("ICEWM_PRIVCFG"))
{
	setCaption(tr("IceSound Configurator " VERSION));

	QBoxLayout *topLayout = new QHBoxLayout(this, 0, -1, "topLayout");
	CHECK_PTR(topLayout);
	
	/**
	 * A block of controls for Ice Sounds Events.
	 */
	QVBox *vb = new QVBox(this, "vbLeft");
	CHECK_PTR(vb);
	topLayout->addWidget(vb);
	
	QLabel *lb = new QLabel(tr("&Ice Sound's dir"), vb, "lbice");
	CHECK_PTR(lb);

	QHBox *hb = new QHBox(vb, "hbl") ;
	CHECK_PTR(hb);

	edIceSoundsDir = new QLineEdit(hb);
	CHECK_PTR(edIceSoundsDir);
	QToolTip::add(edIceSoundsDir, tr(" The <b>IceWM</b> sounds' dir. "
			"You can specify this dir as the first argument of command line"));
	lb->setBuddy(edIceSoundsDir);
	
	QPushButton *pb = new QPushButton("...", hb, "pb");
	CHECK_PTR(pb);
	QToolTip::add(pb, tr(" Press this button for choosing <b>IceWM</b> sounds' dir."));
	connect(pb, SIGNAL(clicked()), SLOT(slotGetIceSoundDir()));

	lvIceSoundEvents = new QListView(vb);
	CHECK_PTR(lvIceSoundEvents);
	QToolTip::add(lvIceSoundEvents,
		   tr(" The list of <b>IceWM</b>'s events."));
	lvIceSoundEvents->addColumn(tr("Events"));
	lvIceSoundEvents->setSorting(-1);
	
	/**
	 * A block of controls for any sounds.
	 */
	vb = new QVBox(this, "vbMed");
	CHECK_PTR(vb);
	topLayout->addWidget(vb);
	
	lb = new QLabel(tr("&Sound's dir"), vb, "lbsounds");
	CHECK_PTR(lb);
	
	hb = new QHBox(vb, "hbm") ;
	CHECK_PTR(hb);

	edSoundsDir = new QLineEdit(hb);
	CHECK_PTR(edSoundsDir);
	QToolTip::add(edSoundsDir, tr(" The sounds' dir. "
		"You can specify this dir as the second argument of command line"));
	lb->setBuddy(edSoundsDir);
	
	pb = new QPushButton("...", hb, "pb");
	CHECK_PTR(pb);
	QToolTip::add(pb,
			tr(" Press this button for choosing directory with sounds."));
	connect(pb, SIGNAL(clicked()), SLOT(slotGetSoundDir()));

	lbFiles = new QListBox(vb);
	CHECK_PTR(lbFiles);

	/**
	 * A block of buttons.
	 */
	vb = new QVBox(this, "vbRight");
	CHECK_PTR(vb);
	topLayout->addWidget(vb);

	pb = new QPushButton(tr("&Play Events sound"), vb);
	CHECK_PTR(pb);
	QToolTip::add(pb, tr("Press this button for playing a selected event."));
	connect(pb, SIGNAL(clicked()), SLOT(slotTestIceSound()));
	
	pb = new QPushButton(tr("&Reload Sound's Dir"), vb);
	CHECK_PTR(pb);
	QToolTip::add(pb, tr("Press this button for reloading a sound's dir."));
	connect(pb, SIGNAL(clicked()), SLOT(slotReloadSounds()));
	
	pb = new QPushButton(tr("&Test sound"), vb);
	CHECK_PTR(pb);
	QToolTip::add(pb, tr("Press this button for playing a selected sound."));
	connect(pb, SIGNAL(clicked()), SLOT(slotTestSound()));
	
	pb = new QPushButton(tr("&Set"), vb);
	CHECK_PTR(pb);
	QToolTip::add(pb,
		   	tr("Press this key for assotiating selected wav-file"
				" to selected <b>IceWM</b> event"));
	connect(pb, SIGNAL(clicked()), SLOT(slotSet()));
	
	pb = new QPushButton(tr("&Unset"), vb);
	CHECK_PTR(pb);
	QToolTip::add(pb,
			tr("Press this key if you want to delete sound for selected event "));
	connect(pb, SIGNAL(clicked()), SLOT(slotUnset()));
	
	pb = new QPushButton(tr("&Help"), vb);
	CHECK_PTR(pb);
	QToolTip::add(pb,
			tr("Shows small help about using of <i>IceWM Sound Configurator.</i>"));
	connect(pb, SIGNAL(clicked()), SLOT(slotHelp()));
	
	pb = new QPushButton(tr("&About"), vb);
	CHECK_PTR(pb);
	connect(pb, SIGNAL(clicked()), SLOT(slotHelpAbout()));
	
	pb = new QPushButton(tr("&Close"), vb);
	CHECK_PTR(pb);
	connect(pb, SIGNAL(clicked()), qApp, SLOT(quit()));

	if(icewmCfgDir.isNull())
		icewmCfgDir = QDir::homeDirPath() + "/.icewm";
	icewmCfgDir += "/sounds/";

	edIceSoundsDir->setText(icewmCfgDir);

	for(int i = sizeof(gui_events)/sizeof(gui_events[0]) - 1; i >= 0; i--)
		new QListViewItem(lvIceSoundEvents, gui_events[i].name);
	lvIceSoundEvents->setMinimumWidth(lvIceSoundEvents->sizeHint().width());

	if(qApp->argc() > 1)
		edIceSoundsDir->setText(qApp->argv()[1]);
	if(qApp->argc() > 2)
	{
		edSoundsDir->setText(qApp->argv()[2]);
		slotReloadSounds();
	} /* 	if(qApp->argc() > 2) */
}//constructor

IcesndcfgApp::~IcesndcfgApp()
{
}//destructor

void IcesndcfgApp::slotHelpAbout()
{
	QMessageBox::about(this,tr("About..."),
       tr("Icesndcfg\nVersion " VERSION "\n(c) 2003-2004 by Vadim A. Khohlov") );
}

/**
* Gets directory of ice sounds.
*/
void
IcesndcfgApp::slotGetIceSoundDir()
{
	QString dirName(QFileDialog::getExistingDirectory(icewmCfgDir,
				this, "geticedir", tr("Choose a directory"), TRUE));
	if(!dirName.isNull())
		edIceSoundsDir->setText(dirName);
} /* IcesndcfgApp ::slotGetIceSoundDir() */

/**
* Gets directory with any sounds.
*/
void
IcesndcfgApp::slotGetSoundDir()
{
	QString dirName(QFileDialog::getExistingDirectory(edSoundsDir->text(),
				this, "geticedir", tr("Choose a directory"), TRUE));
	if(!dirName.isNull())
	{
		edSoundsDir->setText(dirName);
		slotReloadSounds();
	} /* 	if(!dirName.isNull()) */
} /* IcesndcfgApp ::slotGetSoundDir() */

/**
* Reloads files with sounds.
*/
void
IcesndcfgApp::slotReloadSounds()
{
	QDir dir(edSoundsDir->text());

	const QFileInfoList *lst = dir.entryInfoList("*.wav", QDir::Files);
	if(!lst)
		QMessageBox::warning(this, tr("Error!"), tr("Can't read ")+dir.path(),
				QMessageBox::Ok, QMessageBox::NoButton);
	else
	{
		QFileInfoListIterator it(*lst);
		QFileInfo *fi;
		lbFiles->clear();
		while((fi = it.current()) != 0)
		{
			lbFiles->insertItem(fi->fileName());
			++it;
		} /* 		while((fi = it.current()) != 0) */
		lbFiles->setMinimumWidth(lbFiles->sizeHint().width());
	} /* else if(!list)*/
} /* IcesndcfgApp ::slotReloadSounds() */

/**
* Test selected Ice Sound.
*/
void
IcesndcfgApp::slotTestIceSound()
{
	QListViewItem *curIt = lvIceSoundEvents->currentItem();
	if(curIt)
	{
		copyFile(QFile::decodeName("/dev/dsp"), 
			edIceSoundsDir->text() + "/" + curIt->text(0) + ".wav");
	} /* 	if(curIt) */
} /* IcesndcfgApp ::slotTestIceSound() */

/**
* Removes sound for selected event.
*/
void
IcesndcfgApp::slotUnset()
{
	QListViewItem *curIt = lvIceSoundEvents->currentItem();
	if(curIt)
		QFile::remove(edIceSoundsDir->text() + "/" + curIt->text(0) + ".wav");
} /* IcesndcfgApp ::slotUnset( */

/**
* Plays selected sound file;
*/
void
IcesndcfgApp::slotTestSound()
{
	if(lbFiles->currentItem() > -1)
	{
		copyFile(QFile::decodeName("/dev/dsp"), 
			edSoundsDir->text() + "/" + lbFiles->currentText());
	} /* 	if(lbFiles->currentItem() > -1) */	
} /* IcesndcfgApp ::slotTestSound() */

/**
* shows small help dialog.
*/
void
IcesndcfgApp::slotHelp()
{
	QMessageBox::about(this,tr("Help..."),
       tr("\tThis is a small utility for configuration IceSound. \n\n"
		  " It allows you to set any wav-file for IceWM guievents very easy. \n"
		  "\tUsage:\n\t\t$ icesndcfg [path-to-IceWM-sounds [path-to-sound-files]]\n\t"
		  "For example:\n\t\t$ icesndcfg ~/.icewm/sounds /usr/share/sounds\n"
		  "Or you can insert into your ~/.icewm/menu something like this:\n"
		  " prog \"IceSound Config.\" icon icesndcfg /home/Vadim/.icewm/sounds"
		  " /usr/share/sounds \n\n"
		  "\tI hope that using of this utility is evident (see hints for controls). "));
} /* IcesndcfgApp ::slotHelp() */

/**
* Set selected ice event to selected sound.
*/
void
IcesndcfgApp::slotSet()
{
	QListViewItem *curIt = lvIceSoundEvents->currentItem();
	if(lbFiles->currentItem() > -1 && curIt)
	{
		copyFile(edIceSoundsDir->text() + curIt->text(0) + ".wav",
			edSoundsDir->text() + lbFiles->currentText());
	} /* 	if(lbFiles->currentItem() > -1) */	
} /* IcesndcfgApp ::slotSet() */

/**
* Copies file fSrc to file fDst.
*/
void
IcesndcfgApp::copyFile(const QString &fDst, const QString &fSrc)
{
	QFile fileSrc(fSrc);
	QFile fileDst(fDst);

	if(!fileSrc.open(IO_ReadOnly))
	{
		QMessageBox::critical(this, tr("Error!"),
				tr("Can't open %1!").arg(fSrc));
		return;
	} /* 	if(!fileSrc.exists()) */
	
	if(!fileDst.open(IO_WriteOnly | IO_Raw))
	{
		QMessageBox::critical(this, tr("Error!"),
				tr("Can't open %1!").arg(fDst));
		return;
	} /* 	if(!fileSrc.exists()) */

	/* I hope that sound files is not so big. */
	fileDst.writeBlock(fileSrc.readAll());
	
	fileSrc.close();
	fileDst.close();
} /* IcesndcfgApp ::copyFile(const QString &fSrc, const QString &fDst) */
